@@ -1,44 +1,119 @@
-# Copyright (c) 2019 Apex Resource Management Solution Ltd. (ApexRMS). All rights reserved.
-# GPL v.3 License
+# Copyright (c) 2021 Apex Resource Management Solution Ltd. (ApexRMS). All rights reserved.
+# MIT License
 #' @include AAAClassDefinitions.R
 NULL
 
-#' Save datasheet(s)
-#'
-#' Saves datasheets to a SsimLibrary/Project/Scenario.
+#' Save Datasheet(s)
 #'
+#' Saves Datasheets to a \code{\link{SsimLibrary}}, \code{\link{Project}}, or 
+#' \code{\link{Scenario}}.
+#' 
+#' @param ssimObject \code{\link{SsimLibrary}}, \code{\link{Project}}, or 
+#'     \code{\link{Scenario}} object
+#' @param data data.frame, named vector, or list of these. One or more 
+#'     Datasheets to load
+#' @param name character or vector of these. The name(s) of the Datasheet(s) to 
+#'     be saved. If a vector of names is provided, then a list must be provided 
+#'     for the \code{data} argument. Names provided here will override those provided 
+#'     with \code{data} argument's list
+#' @param fileData named list or raster stack. Names are file names (without paths), 
+#'     corresponding to entries in \code{data} The elements are objects containing the 
+#'     data associated with each name. Currently only supports Raster objects 
+#'     as elements
+#' @param append logical. If \code{TRUE}, the incoming data will be appended to the 
+#'     Datasheet if possible.  Default is \code{TRUE} for Project/SsimLibrary-scope Datasheets, 
+#'     and \code{FALSE} for Scenario-scope Datasheets. See 'details' for more information 
+#'     about this argument
+#' @param forceElements logical. If \code{FALSE} (default) a single return message will 
+#'     be returned as a character string. Otherwise it will be returned in a list
+#' @param force logical. If Datasheet scope is Project/SsimLibrary, and \code{append=FALSE}, 
+#'     Datasheet will be deleted before loading the new data. This can also delete 
+#'     other definitions and results, so if \code{force=FALSE} (default) user will be 
+#'     prompted for approval 
+#' @param breakpoint logical. Set to \code{TRUE} when modifying Datasheets in a 
+#' breakpoint function. Default is \code{FALSE}
+#' @param import logical. Set to \code{TRUE} to import the data after saving. 
+#' Default is \code{FALSE}
+#' @param path character.  output path (optional)
+#' 
 #' @details
-#' ssimObject/project/scenario should identify a single ssimObject.
+#' SsimObject/Project/Scenario should identify a single SsimObject.
 #'
-#' If fileData !=NULL, each element of names(fileData) should correspond uniquely to at most one entry in data. If a name is not found in data the element will be ignored with a warning.
-#' If names(fileData) are full filepaths, rsyncrosim will write each object to the corresponding path for subsequent loading by SyncroSim. Note this is generally more time-consuming because the files must be written twice.
-#' If names(fileData) are not filepaths (faster, recommended), rsyncrosim will write each element directly to the appropriate SyncroSim input/output folders.
-#' rsyncrosim will write each element of fileData directly to the appropriate SyncroSim input/output folders.
-#' If fileData != NULL, data should be a dataframe, vector, or list of length 1, not a list of length >1.
+#' If \code{fileData != NULL}, each element of \code{names(fileData)} should correspond uniquely 
+#' to at most one entry in data. If a name is not found in data the element will 
+#' be ignored with a warning. If \code{names(fileData)} are full filepaths, rsyncrosim 
+#' will write each object to the corresponding path for subsequent loading by SyncroSim. 
+#' Note this is generally more time-consuming because the files must be written twice.
+#' If \code{names(fileData)} are not filepaths (faster, recommended), rsyncrosim will 
+#' write each element directly to the appropriate SyncroSim input/output folders.
+#' rsyncrosim will write each element of fileData directly to the appropriate 
+#' SyncroSim input/output folders. If \code{fileData != NULL}, data should be a data.frame, 
+#' vector, or list of length 1, not a list of length >1.
 #' 
 #' About the 'append' argument:
 #' 
 #' \itemize{
-#'   \item A datasheet is a VALIDATION SOURCE if its data can be used to validate column values in a different datasheet.
-#'   \item The append argument will be ignored if the datasheet is a validation source and has a project scope.  In this case the data will be MERGED.
+#'   \item A Datasheet is a VALIDATION SOURCE if its data can be used to validate 
+#'   column values in a different Datasheet.
+#'   \item The \code{append} argument will be ignored if the Datasheet is a validation 
+#'   source and has a Project scope.  In this case the data will be MERGED.
 #' }
 #' 
-#' @param ssimObject SsimLibrary/Project/Scenario.
-#' @param data A dataframe, named vector, or list of these. One or more datasheets to load.
-#' @param name Character or vector of these. The name(s) of the datasheet(s) to be saved. If a vector of names is provided, then a list must be provided for the data argument. Names provided here will override those provided with data argument's list.
-# @param project character or integer. Project name or id. Note integer ids are slightly faster.
-# @param scenario character or integer. Project name or id. Note integer ids are slightly faster.
-#' @param fileData Named list or raster stack. Names are file names (without paths), corresponding to entries in data. The elements are objects containing the data associated with each name. Currently only supports Raster objects as elements.
-#' @param append Logical. If TRUE, the incoming data will be appended to the datasheet if possible.  Default TRUE for project/library-scope datasheets, and FALSE for scenario-scope datasheets. See 'details' for more information about this argument.
-#' @param forceElements Logical. If FALSE (default) a single return message will be returned as a character string. Otherwise it will be returned in a list.
-#' @param force Logical. If datasheet scope is project/library, and append=FALSE, datasheet will be deleted before loading the new data. This can also delete other definitions and results, so user will be prompted for approval unless force=TRUE.
-#' @param breakpoint Set to TRUE when modifying datasheets in a breakpoint function.
-#' @param import Logical. Set to TRUE to import the data after saving.
-#' @param path Character.  An optional output path.
-#' 
 #' @return 
-#' This function invisibly returns a vector or list of logical values for each input: `TRUE` upon success (i.e.successful save)
-#' and `FALSE` upon failure.
+#' Invisibly returns a vector or list of logical values for each 
+#' input: \code{TRUE} upon success (i.e.successful save) and \code{FALSE} upon failure.
+#' 
+#' @examples 
+#' \donttest{
+#' # Install helloworldSpatial package
+#' addPackage("helloworldSpatial")
+#' 
+#' # Set the file path and name of the new SsimLibrary
+#' myLibraryName <- file.path(tempdir(),"testlib_saveDatasheet")
+#' 
+#' # Set the SyncroSim Session, SsimLibrary, Project, and Scenario
+#' mySession <- session()
+#' myLibrary <- ssimLibrary(name = myLibraryName,
+#'                          session = mySession, 
+#'                          package = "helloworldSpatial",
+#'                          template = "example-library")
+#' myProject <- project(myLibrary, project = "Definitions")
+#' myScenario <- scenario(myProject, scenario = "My Scenario")
+#' 
+#' # Get all Datasheet info
+#' myDatasheets <- datasheet(myScenario)
+#' 
+#' # Get a specific Datasheet
+#' myDatasheet <- datasheet(myScenario, name = "RunControl")
+#' 
+#' # Modify Datasheet
+#' myDatasheet$MaximumTimestep <- 10
+#' 
+#' # Save Datasheet
+#' saveDatasheet(ssimObject = myScenario, data = myDatasheet, name = "RunControl")
+#'           
+#' # Import data after saving
+#' saveDatasheet(ssimObject = myScenario, data = myDatasheet, name = "RunControl",
+#'               import = TRUE)
+#'         
+#' # Save the new Datasheet to a specified output path
+#' saveDatasheet(ssimObject = myScenario, data = myDatasheet, name = "RunControl",
+#'               path = tempdir())
+#'               
+#' # Save a raster stack using fileData
+#' # Create a raster stack - add as many raster files as you want here
+#' map1 <- datasheetRaster(myScenario, datasheet = "InputDatasheet",
+#'                         column = "InterceptRasterFile")
+#' inRasters <- raster::stack(map1)
+#' 
+#' # Change the name of the rasters in the input Datasheets to match the stack
+#' inSheet <- datasheet(myScenario, name="InputDatasheet")
+#' inSheet[1,"InterceptRasterFile"] <- names(inRasters)[1]
+#' 
+#' # Save the raster stack to the input Datasheet
+#' saveDatasheet(myScenario, data=inSheet, name="InputDatasheet", 
+#'               fileData=inRasters)
+#' }
 #' 
 #' @export
 setGeneric("saveDatasheet", function(ssimObject, data, name = NULL, fileData = NULL, append = NULL, forceElements = FALSE, force = FALSE, breakpoint = FALSE, import = TRUE, path = NULL) standardGeneric("saveDatasheet"))
@@ -50,6 +125,7 @@ setMethod("saveDatasheet", signature(ssimObject = "character"), function(ssimObj
 
 #' @rdname saveDatasheet
 setMethod("saveDatasheet", signature(ssimObject = "SsimObject"), function(ssimObject, data, name, fileData, append, forceElements, force, breakpoint, import, path) {
+  
   isFile <- NULL
   x <- ssimObject
   if (is.null(append)) {
@@ -133,7 +209,7 @@ setMethod("saveDatasheet", signature(ssimObject = "SsimObject"), function(ssimOb
 
     # convert factors to strings
     for (kk in seq(length.out = ncol(cDat))) {
-      if (class(cDat[[kk]]) == "factor") {
+      if (is.factor(cDat[[kk]])) {
         cDat[[kk]] <- as.character(cDat[[kk]])
       }
     }
@@ -141,7 +217,7 @@ setMethod("saveDatasheet", signature(ssimObject = "SsimObject"), function(ssimOb
     # note deletions must happen before files are written.
     scope <- sheetNames$scope[sheetNames$name == cName]
     if (length(scope) == 0) {
-      sheetNames <- datasheets(x, refresh = TRUE)
+      sheetNames <- datasheets(x, core = TRUE)
       scope <- sheetNames$scope[sheetNames$name == cName]
       if (length(scope) == 0) {
         stop("Name not found in datasheetNames")
@@ -184,7 +260,7 @@ setMethod("saveDatasheet", signature(ssimObject = "SsimObject"), function(ssimOb
       if (is.null(itemNames) || is.na(itemNames) || (length(itemNames) == 0)) {
         stop("names(fileData) must be defined, and each element must correspond uniquely to an entry in data")
       }
-
+    
       sheetInfo <- subset(datasheet(x, summary = TRUE, optional = TRUE), name == cName)
       fileDir <- .filepath(x)
       if (sheetInfo$isOutput) {
